IND_INDICES = set(["BSE CG","SENSEX","BSE CD","NIFTY50 PR 1x INV","BSE IT","METAL",
                "OILGAS","NIFTY50 PR 2x LEV","BSEIPO","GREENX","POWER","NIFTY50 TR 1x INV",
                "CARBON","BASMTR","CDGS","NIFTY50 TR 2x LEV","BSEFMC","BSE HC","ALLCAP",
                "NIFTY50 TR 2x LEV","REALTY","SMEIPO","DOL30","NIFTY Mid LIQ 15","LRGCAP",
                "MIDSEL","SMLSEL","NIFTY100 LIQ 15","SNXT50","SNSX50","NIFTY 50","NIFTY Quality 30",
                "NIFTY BANK","NIFTY NEXT 50","DOL100","NIFTY MIDCAP 50","NIFTY 100","NIFTY 200",
                "NIFTY 500","NIFTY FIN SERVICE","NIFTY AUTO","NIFTY FMCG","NIFTY IT","NIFTY COMMODITIES",
                "NIFTY MEDIA","NIFTY METAL","NIFTY PHARMA","NIFTY CONSUMPTION","NIFTY PSU BANK",
                "NIFTY PVT BANK","NIFTY REALTY","NIFTY GROWSECT 15","NIFTY CPSE","NIFTY ENERGY",
                "NIFTY INFRA","NIFTY DIV OPPS 50","NIFTY MNC","NIFTY PSE","NIFTY SERV SECTOR",
                "NIFTY MID100 FREE","DOL200","TECK","BSEPSU","NIFTY SML100 FREE","AUTO","BANKEX",
                "INDIA VIX","NIFTY50 VALUE 20","NIFTY","BANKNIFTY","FINNIFTY"
                ])